#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "Record.h"
#include "FileReader.h"

//---------------------------------------------------------------
// storeOpen()
//   Must be called before any recordRead() calls are made.
//   First argument is the name of a file presumed to have
//     appropriately formatted record contents.
//   Second argument is a pointer to a RecordAddress to
//     be filled with the address of the root record.
//   Returns:  0 for success
//             non-zero for failure
//--------------------------------------------------------------

int storeOpen( char* filename, RecordAddress* rootAddress ) {
  int rcode;

  // check for a common errors (?), null passed for one of the args
  if ( !filename || !rootAddress ) return -1;

  rcode = fileOpen( filename );
  if ( rcode ) return rcode; // bail if an error occurred

  // the root is stored in the first four bytes of the file
  fileRead( 0, sizeof(*rootAddress), rootAddress);
  return 0;
}

//---------------------------------------------------
// storeClose()
//   Should be invoked after done with the store.
//   Gives it a chance to do any clean-up it might need.
//----------------------------------------------------

void storeClose() {
  fileClose();
}

//---------------------------------------------------
// storeRead()
//   Reads the record at the (byte) address given.
//   Input is a byte address (not a record number!)
//   and a pointer to a Record where the data should
//   be put.
//   Returns: 0 for success
//            non-zero for failure
//---------------------------------------------------

int storeRead( Record* pRecord, RecordAddress addr ) {

  // sanity check -- address should be a multiple of the record length
  if ( addr % RECORDLENGTH ) return -1;

  return fileRead( addr, RECORDLENGTH, pRecord );
}
